@@ -4,9 +4,9 @@ mod handshake_message_certificate_verify_test;
 use super::*;
 use crate::signature_hash_algorithm::*;
 
-use std::io::{Read, Write};
-
+use anyhow::Result;
 use byteorder::{BigEndian, ReadBytesExt, WriteBytesExt};
+use std::io::{Read, Write};
 
 #[derive(Clone, Debug, PartialEq)]
 pub struct HandshakeMessageCertificateVerify {
@@ -26,7 +26,7 @@ impl HandshakeMessageCertificateVerify {
         1 + 1 + 2 + self.signature.len()
     }
 
-    pub fn marshal<W: Write>(&self, writer: &mut W) -> Result<(), Error> {
+    pub fn marshal<W: Write>(&self, writer: &mut W) -> Result<()> {
         writer.write_u8(self.hash_algorithm as u8)?;
         writer.write_u8(self.signature_algorithm as u8)?;
         writer.write_u16::<BigEndian>(self.signature.len() as u16)?;
@@ -35,7 +35,7 @@ impl HandshakeMessageCertificateVerify {
         Ok(writer.flush()?)
     }
 
-    pub fn unmarshal<R: Read>(reader: &mut R) -> Result<Self, Error> {
+    pub fn unmarshal<R: Read>(reader: &mut R) -> Result<Self> {
         let hash_algorithm = reader.read_u8()?.into();
         let signature_algorithm = reader.read_u8()?.into();
         let signature_length = reader.read_u16::<BigEndian>()? as usize;
